#ifndef UTILS_STRINGMAP_H
#define UTILS_STRINGMAP_H

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

// https://stackoverflow.com/questions/466204/rounding-up-to-next-power-of-2
// https://stackoverflow.com/questions/1322510/given-an-integer-how-do-i-find-the-next-largest-power-of-two-using-bit-twiddlin/1322548#1322548
static inline uint64_t utils_nearest_bigger_power_of_2_u64(uint64_t value)
{
	value--;
	value |= value >> 1;
	value |= value >> 2;
	value |= value >> 4;
	value |= value >> 8;
	value |= value >> 16;
	value |= value >> 32;
	value++;
	return value;
}

// https://stackoverflow.com/questions/7666509/hash-function-for-string
// http://www.cse.yorku.ca/~oz/hash.html
static inline size_t string_hash_djb2(const unsigned char* value, size_t length)
{
	size_t hash = 5381;
	for (size_t i = 0; i < length && value[i] != '\0'; ++i)
		hash = ((hash << 5) + hash) + value[i];
	return hash;
}

typedef struct StringMap StringMap;

StringMap* stringmap_new(void);
void stringmap_delete(StringMap* map);
size_t* stringmap_get(const StringMap* map, const char* key, size_t key_length);
bool stringmap_has(const StringMap* map, const char* key, size_t key_length);
void stringmap_set(
	StringMap* map, const char* key, size_t key_length, size_t value
);
void stringmap_reserve(StringMap* map, size_t minimum_size);
void stringmap_remove(StringMap* map, const char* key, size_t key_length);
void stringmap_clean(StringMap* map);
void stringmap_shrink(StringMap* map);
void stringmap_clean_and_shrink(StringMap* map);

#endif
